@@ -27,45 +27,58 @@ extern "C" {
 #endif
 
 /**
- * Sets the mixer device and channel.  The device name is of the form
- * devicename:channelname.  The default is /dev/mixer:line.
+ * Sets the mixer device and channel.
+ * All interfaces are scanned until one succeeds.
  */
 void mixer_set_device( const char *devname );
 
+struct mixer {
+/**
+ * Sets the mixer device and channel.
+ */
+int (* set_device)( const char *devname );
+
 /**
  * Sets the initial state of the mixer device.
  */
-void mixer_set_state( int ismuted, int unmute_volume );
+void (* set_state)( int ismuted, int unmute_volume );
 
 /**
  * Returns the current volume setting.
  */
-int mixer_get_volume( void );
+int (* get_volume)( void );
 
 /**
  * Returns the volume that would be used to restore the unmute state.
  */
-int mixer_get_unmute_volume( void );
+int (* get_unmute_volume)( void );
 
 /**
  * Tunes the relative volume.
  */
-int mixer_set_volume( int percentdiff );
+int (* set_volume)( int percentdiff );
 
 /**
  * Sets the mute state.
  */
-void mixer_mute( int mute );
+void (* mute)( int mute );
 
 /**
  * Returns true if the mixer is muted.
  */
-int mixer_ismute( void );
+int (* ismute)( void );
 
 /**
  * Closes the mixer device if it is open.
  */
-void mixer_close_device( void );
+void (* close_device)( void );
+};
+
+#pragma weak alsa_mixer
+extern struct mixer alsa_mixer;
+#pragma weak oss_mixer
+extern struct mixer oss_mixer;
+extern struct mixer *mixer;
 
 #ifdef __cplusplus
 };
