@@ -59,6 +59,7 @@ struct config_s
     int squarepixels;
     int debug;
     int fullscreen;
+    int borderless;
     int alwaysontop;
     int priority;
     int ntsc_mode;
@@ -120,6 +121,9 @@ struct config_s
 
     char *config_filename;
     xmlDocPtr doc;
+
+    char *alsa_inputdev;
+    char *alsa_outputdev;
 };
 
 static unsigned int parse_colour( const char *str )
@@ -320,6 +324,10 @@ static void parse_option( config_t *ct, xmlNodePtr node )
             ct->fullscreen = atoi( curval );
         }
 
+        if( !xmlStrcasecmp( name, BAD_CAST "Borderless" ) ) {
+            ct->borderless = atoi( curval );
+        }
+
         if( !xmlStrcasecmp( name, BAD_CAST "AlwaysOnTop" ) ) {
             ct->alwaysontop = atoi( curval );
         }
@@ -402,6 +410,17 @@ static void parse_option( config_t *ct, xmlNodePtr node )
             if( ct->mixerdev ) free( ct->mixerdev );
             ct->mixerdev = strdup( curval );
         }
+
+        if( !xmlStrcasecmp( name, BAD_CAST "AlsaInputDev" ) ) {
+            if( ct->alsa_inputdev ) free( ct->alsa_inputdev );
+            ct->alsa_inputdev = strdup( curval );
+        }
+
+        if( !xmlStrcasecmp( name, BAD_CAST "AlsaOutputDev" ) ) {
+            if( ct->alsa_outputdev ) free( ct->alsa_outputdev );
+            ct->alsa_outputdev = strdup( curval );
+        }
+
     }
 
     if( name ) xmlFree( name );
@@ -619,6 +638,7 @@ static void print_usage( char **argv )
               "                             (defaults to 720 pixels).\n"), stderr );
     lfputs( _("  -k, --slave                Disables input handling in tvtime (slave mode).\n"), stderr );
     lfputs( _("  -m, --fullscreen           Start tvtime in fullscreen mode.\n"), stderr );
+    lfputs( _("  -l, --borderless           Start tvtime without a window border.\n"), stderr );
     lfputs( _("  -M, --window               Start tvtime in window mode.\n"), stderr );
     lfputs( _("  -n, --norm=NORM            The norm to use for the input.  tvtime supports:\n"
               "                             NTSC, NTSC-JP, SECAM, PAL, PAL-Nc, PAL-M,\n"
@@ -629,9 +649,11 @@ static void print_usage( char **argv )
     lfputs( _("  -l, --xmltvlanguage=LANG   Use XMLTV data in given language, if available.\n"), stderr );
     lfputs( _("  -v, --verbose              Print debugging messages to stderr.\n"), stderr );
     lfputs( _("  -X, --display=DISPLAY      Use the given X display to connect to.\n"), stderr );
-    lfputs( _("  -x, --mixer=DEVICE[:CH]    The mixer device and channel to control.\n"
-              "                             (defaults to /dev/mixer:line)\n\n"
-              "                             Valid channels are:\n"
+    lfputs( _("  -x, --mixer=<DEVICE[:CH]>|<DEVICE/CH>\n"
+              "                             The mixer device and channel to control. The first\n"
+              "                             variant sets the OSS mixer the second one ALSA.\n"
+              "                             (defaults to default/Master)\n\n"
+              "                             Valid channels for OSS are:\n"
               "                                 vol, bass, treble, synth, pcm, speaker, line,\n"
               "                                 mic, cd, mix, pcm2, rec, igain, ogain, line1,\n"
               "                                 line2, line3, dig1, dig2, dig3, phin, phout,\n"
@@ -670,6 +692,7 @@ static void print_config_usage( char **argv )
     lfputs( _("  -I, --inputwidth=SAMPLING  Horizontal resolution of input\n"
               "                             (defaults to 720 pixels).\n"), stderr );
     lfputs( _("  -m, --fullscreen           Start tvtime in fullscreen mode.\n"), stderr );
+    lfputs( _("  -l, --borderless           Start tvtime without a window border.\n"), stderr );
     lfputs( _("  -M, --window               Start tvtime in window mode.\n"), stderr );
     lfputs( _("  -n, --norm=NORM            The norm to use for the input.  tvtime supports:\n"
               "                             NTSC, NTSC-JP, SECAM, PAL, PAL-Nc, PAL-M,\n"
@@ -677,13 +700,23 @@ static void print_config_usage( char **argv )
     lfputs( _("  -R, --priority=PRI         Sets the process priority to run tvtime at.\n"), stderr );
     lfputs( _("  -t, --xmltv=FILE           Read XMLTV listings from the given file.\n"), stderr );
     lfputs( _("  -l, --xmltvlanguage=LANG   Use XMLTV data in given language, if available.\n"), stderr );
-    lfputs( _("  -x, --mixer=DEVICE[:CH]    The mixer device and channel to control.\n"
-              "                             (defaults to /dev/mixer:line)\n\n"
-              "                             Valid channels are:\n"
+    lfputs( _("  -x, --mixer=<DEVICE[:CH]>|<DEVICE/CH>\n"
+              "                             The mixer device and channel to control. The first\n"
+              "                             variant sets the OSS mixer the second one ALSA.\n"
+              "                             (defaults to default/Master)\n\n"
+              "                             Valid channels for OSS are:\n"
               "                                 vol, bass, treble, synth, pcm, speaker, line,\n"
               "                                 mic, cd, mix, pcm2, rec, igain, ogain, line1,\n"
               "                                 line2, line3, dig1, dig2, dig3, phin, phout,\n"
               "                                 video, radio, monitor\n"), stderr );
+    lfputs( _("  -p, --alsainputdev=DEV     Specifies an ALSA device to read input on\n"
+              "                                 Examples:\n"
+              "                                          hw:1,0\n"
+              "                                          disabled\n"), stderr );
+    lfputs( _("  -P, --alsaoutputdev=DEV    Specifies an ALSA device to write output to\n"
+              "                                 Examples:\n"
+              "                                          hw:0,0\n"
+              "                                          disabled\n"), stderr );
 }
 
 static void print_scanner_usage( char **argv )
@@ -715,6 +748,7 @@ config_t *config_new( void )
     ct->squarepixels = 1;
     ct->debug = 0;
     ct->fullscreen = 0;
+    ct->borderless = 0;
     ct->alwaysontop = 0;
     ct->priority = -10;
     ct->ntsc_mode = 0;
@@ -764,7 +798,7 @@ config_t *config_new( void )
 
     ct->uid = getuid();
 
-    ct->mixerdev = strdup( "/dev/mixer:line" );
+    ct->mixerdev = strdup( "default/Master" );
 
     ct->deinterlace_method = strdup( "GreedyH" );
     ct->check_freq_present = 1;
@@ -782,6 +816,9 @@ config_t *config_new( void )
     ct->config_filename = 0;
     ct->doc = 0;
 
+    ct->alsa_inputdev = strdup( "hw:1,0" );
+    ct->alsa_outputdev = strdup( "hw:0,0" );
+
     /* Default key bindings. */
     ct->keymap[ 0 ] = TVTIME_NOCOMMAND;
     ct->keymap[ I_ESCAPE ] = TVTIME_QUIT;
@@ -804,6 +841,7 @@ config_t *config_new( void )
     ct->keymap[ I_F6 ] = TVTIME_PICTURE_DOWN;
     ct->keymap[ I_F7 ] = TVTIME_PICTURE_UP;
     ct->keymap[ 'd' ] = TVTIME_SHOW_STATS;
+    ct->keymap[ 'g' ] = TVTIME_SHOW_EPG;
     ct->keymap[ 'a' ] = TVTIME_TOGGLE_ASPECT;
     ct->keymap[ 'f' ] = TVTIME_TOGGLE_FULLSCREEN;
     ct->keymap[ 'i' ] = TVTIME_TOGGLE_INPUT;
@@ -840,7 +878,7 @@ config_t *config_new( void )
     ct->keymapmenu[ 'q' ] = TVTIME_MENU_EXIT;
     ct->keymapmenu[ I_ESCAPE ] = TVTIME_MENU_EXIT;
     ct->buttonmapmenu[ 1 ] = TVTIME_MENU_ENTER;
-    ct->buttonmapmenu[ 3 ] = TVTIME_MENU_ENTER;
+    ct->buttonmapmenu[ 3 ] = TVTIME_MENU_EXIT;
     ct->buttonmapmenu[ 4 ] = TVTIME_MENU_UP;
     ct->buttonmapmenu[ 5 ] = TVTIME_MENU_DOWN;
 
@@ -891,6 +929,7 @@ int config_parse_tvtime_command_line( config_t *ct, int argc, char **argv )
         { "mixer", 1, 0, 'x' },
         { "showdrops", 0, 0, 's' },
         { "fullscreen", 0, 0, 'm' },
+        { "borderless", 0, 0, 'l' },
         { "window", 0, 0, 'M' },
         { "slave", 0, 0, 'k' },
         { "widescreen", 0, 0, 'a' },
@@ -907,17 +946,26 @@ int config_parse_tvtime_command_line( config_t *ct, int argc, char **argv )
     int c;
 
     if( argc ) {
-        while( (c = getopt_long( argc, argv, "aAhkmMsSvF:r:g:I:d:b:i:c:n:D:f:x:X:t:l:Qg:",
+        while( (c = getopt_long( argc, argv, "aAhkmMsSvV:F:r:g:I:d:b:i:c:n:D:f:x:X:t:Ll:Qg:",
                 long_options, &option_index )) != -1 ) {
             switch( c ) {
             case 'a': ct->aspect = 1; break;
             case 'A': ct->aspect = 0; break;
             case 'k': ct->slave_mode = 1; break;
             case 'm': ct->fullscreen = 1; break;
+            case 'L': ct->borderless = 1; break;
             case 'M': ct->fullscreen = 0; break;
             case 's': ct->debug = 1; break;
             case 'S': saveoptions = 1; break;
             case 'v': ct->verbose = 1; break;
+            case 'V': if( !optarg ) {
+                          fprintf( stdout, "UnmuteVolume:%d\n",
+                                   config_get_unmute_volume( ct ) );
+                      } else {
+                          ct->unmute_volume = atoi( optarg );
+                          //mixer_set_volume ( 0, ct->unmute_volume );
+                      }
+                      break;
             case 't': if( ct->xmltvfile ) { free( ct->xmltvfile ); }
                       ct->xmltvfile = expand_user_path( optarg ); break;
             case 'l': if( ct->xmltvlanguage ) { free( ct->xmltvlanguage ); }
@@ -990,6 +1038,9 @@ int config_parse_tvtime_command_line( config_t *ct, int argc, char **argv )
         snprintf( tempstring, sizeof( tempstring ), "%d", ct->fullscreen );
         config_save( ct, "Fullscreen", tempstring );
 
+        snprintf( tempstring, sizeof( tempstring ), "%d", ct->borderless );
+        config_save( ct, "Borderless", tempstring );
+
         snprintf( tempstring, sizeof( tempstring ), "%d", ct->verbose );
         config_save( ct, "Verbose", tempstring );
 
@@ -1032,12 +1083,15 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
         { "device", 2, 0, 'd' },
         { "mixer", 1, 0, 'x' },
         { "fullscreen", 0, 0, 'm' },
+        { "borderless", 0, 0, 'L' },
         { "window", 0, 0, 'M' },
         { "widescreen", 0, 0, 'a' },
         { "nowidescreen", 0, 0, 'A' },
         { "xmltv", 2, 0, 't' },
         { "xmltvlanguage", 2, 0, 'l' },
         { "priority", 2, 0, 'R' },
+        { "alsainputdev", 2, 0, 'p' },
+        { "alsaoutputdev", 2, 0, 'P' },
         { 0, 0, 0, 0 }
     };
     int option_index = 0;
@@ -1049,12 +1103,14 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
         return 0;
     }
 
-    while( (c = getopt_long( argc, argv, "aAhmMF:g:I:d:b:i:c:n:D:f:x:t:l:R:",
+    while( (c = getopt_long( argc, argv,
+            "aAhmMF:g:I:d:b:i:c:n:D:f:x:t:Ll:R:p:P",
             long_options, &option_index )) != -1 ) {
         switch( c ) {
         case 'a': ct->aspect = 1; break;
         case 'A': ct->aspect = 0; break;
         case 'm': ct->fullscreen = 1; break;
+        case 'L': ct->borderless = 1; break;
         case 'M': ct->fullscreen = 0; break;
         case 'F': if( ct->config_filename ) free( ct->config_filename );
                   filename_specified = 1;
@@ -1134,6 +1190,22 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
                       ct->priority = atoi( optarg );
                   }
                   break;
+        case 'p': if( !optarg ) {
+                      fprintf( stdout, "AlsaInputDevice:%s\n", 
+			       config_get_alsa_inputdev( ct ) );
+                  } else {
+                      free( ct->alsa_inputdev );
+                      ct->alsa_inputdev = strdup( optarg );
+                  }
+                  break;
+        case 'P': if( !optarg ) {
+                      fprintf( stdout, "AlsaOutputDevice:%s\n", 
+			       config_get_alsa_outputdev( ct ) );
+                  } else {
+                      free( ct->alsa_outputdev );
+                      ct->alsa_outputdev = strdup( optarg );
+                  }
+                  break;
         default:
             print_config_usage( argv );
             return 0;
@@ -1174,6 +1246,9 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
         snprintf( tempstring, sizeof( tempstring ), "%d", ct->fullscreen );
         config_save( ct, "Fullscreen", tempstring );
 
+        snprintf( tempstring, sizeof( tempstring ), "%d", ct->borderless );
+        config_save( ct, "Borderless", tempstring );
+
         snprintf( tempstring, sizeof( tempstring ), "%d", ct->verbose );
         config_save( ct, "Verbose", tempstring );
 
@@ -1199,6 +1274,9 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
 
         snprintf( tempstring, sizeof( tempstring ), "%d", ct->priority );
         config_save( ct, "ProcessPriority", tempstring );
+
+        config_save( ct, "AlsaInputDev", ct->alsa_inputdev );
+        config_save( ct, "AlsaOutputDev", ct->alsa_outputdev );
     }
 
     return 1;
@@ -1261,6 +1339,8 @@ void config_free_data( config_t *ct )
     if( ct->vbidev ) free( ct->vbidev );
     if( ct->config_filename ) free( ct->config_filename );
     if( ct->deinterlace_method ) free( ct->deinterlace_method );
+    if( ct->alsa_inputdev ) free( ct->alsa_inputdev );
+    if( ct->alsa_outputdev ) free( ct->alsa_outputdev );
 }
 
 void config_delete( config_t *ct )
@@ -1486,6 +1566,11 @@ int config_get_fullscreen( config_t *ct )
     return ct->fullscreen;
 }
 
+int config_get_borderless( config_t *ct )
+{
+    return ct->borderless;
+}
+
 int config_get_alwaysontop( config_t *ct )
 {
     return ct->alwaysontop;
@@ -1660,4 +1745,12 @@ int config_get_square_pixels( config_t *ct )
 {
     return ct->squarepixels;
 }
+const char *config_get_alsa_inputdev( config_t *ct )
+{
+    return ct->alsa_inputdev;
+}
 
+const char *config_get_alsa_outputdev( config_t *ct )
+{
+    return ct->alsa_outputdev;
+}
