@@ -148,6 +148,7 @@ int config_get_verbose( config_t *ct );
 int config_get_send_fields( config_t *ct );
 int config_get_fullscreen_position( config_t *ct );
 int config_get_debug( config_t *ct );
+int config_get_borderless( config_t *ct );
 const char *config_get_geometry( config_t *ct );
 int config_get_inputwidth( config_t *ct );
 int config_get_aspect( config_t *ct );
@@ -192,6 +193,8 @@ int config_get_muted( config_t *ct );
 int config_get_mute_on_exit( config_t *ct );
 int config_get_show_taglines( config_t *ct );
 int config_get_square_pixels( config_t *ct );
+const char *config_get_alsa_inputdev( config_t *ct );
+const char *config_get_alsa_outputdev( config_t *ct );
 
 #ifdef __cplusplus
 };
